float4x4 wvp : WorldViewProjection;
float4x4 world : World;
float4x4 worldI : WorldI;
float4x4 viewI : ViewI;
float4x4 view : View;

struct appdata {
    float4 Position	 : POSITION;
    float4 texcoord0 : TEXCOORD0;
    float4 texcoord1 : TEXCOORD1;
    float4 texcoord2 : TEXCOORD2;
    float4 normal	   : NORMAL;
};

struct vertexOutput {
    float4 color     :   COLOR0;
    float4 Position  : POSITION;
    float4 texcoord0 : TEXCOORD0;
    float4 texcoord1 : TEXCOORD1;
    float4 texcoord2 : TEXCOORD2;
    float4 texcoord3 : TEXCOORD3;
    float4 texcoord4 : TEXCOORD4;
    float4 texcoord5 : TEXCOORD5;
    float4 texcoord6 : TEXCOORD6;
    float4 texcoord7 : TEXCOORD7;
};

struct pixelOutput {
  float4 col : COLOR;
};

vertexOutput mainVS(appdata IN,
    uniform float4x4 WorldViewProj,
    uniform float4x4 World,
    uniform float4x4 WorldI,
    uniform float4x4 ViewI,
    uniform float4x4 View,
    uniform float minRefl, 
    uniform float maxRefl, 
    uniform float fresnelEXP,
    uniform float3 lightPosition
) 
{
    vertexOutput OUT;

    OUT.Position = mul(WorldViewProj, IN.Position);
    
    float3x3 Obj2Tangent   = float3x3(IN.texcoord1.xyz, IN.texcoord2.xyz, IN.normal.xyz);
    float3x3 Tangent2World = mul((float3x3)World, transpose(Obj2Tangent));

    lightPosition     = mul(ViewI, float4(lightPosition, 1.0)).xyz;
    float3 eyepos     = mul(ViewI, float4(0.0, 0.0, 0.0, 1.0)).xyz;
    float3 worldpos   = mul(World, IN.Position).xyz;
    float3 normal     = normalize(mul(IN.normal, WorldI).xyz);
    float3 viewVector = worldpos - eyepos; // don't normalize cause of interpolation error!
    float3 reflVector = reflect(viewVector, normal);
    float3 lightVector = worldpos - lightPosition.xyz;

    // compute fresnel equation
    float F_reflect  = minRefl + (maxRefl-minRefl)*pow(1.0 - dot(-normalize(viewVector), normal), fresnelEXP);
    float F_lighting = minRefl + (maxRefl-minRefl)*pow(1.0 - dot(-normalize(lightVector), -normalize(viewVector)), fresnelEXP);

    OUT.texcoord1.xyz = Tangent2World[0].xyz;
    OUT.texcoord2.xyz = Tangent2World[1].xyz;
    OUT.texcoord3.xyz = Tangent2World[2].xyz;

    OUT.texcoord0 = IN.texcoord0;
    OUT.texcoord4.xyz = reflVector;
    OUT.texcoord4.w   = F_reflect;
    
    OUT.texcoord5.xyz = lightVector;
    OUT.texcoord5.w   = F_lighting;

    OUT.texcoord6.xyz = viewVector;
    OUT.texcoord7.xyz = normal;
 
    return OUT;
}

pixelOutput fresnel( vertexOutput IN, uniform samplerCUBE EnvironmentMap,  
                     uniform sampler2D NormalMap_Reflect, uniform float bumpy_reflect, 
                     uniform float3 color, uniform float shininess,
                     uniform float refl, uniform float diffuse, uniform float specular, uniform float col)
{
    pixelOutput OUT; 

    float3 normal         = normalize(IN.texcoord7.xyz);
    float3 viewVector     = normalize(IN.texcoord6.xyz);
    float3 lightVector    = normalize(IN.texcoord5.xyz);

    float3x3 tangent2World = float3x3(IN.texcoord1.xyz, IN.texcoord2.xyz, IN.texcoord3.xyz);

    float3 tangent_normal_reflect = (tex2D(NormalMap_Reflect, IN.texcoord0.xy).xyz - 0.5) * 2.0;
    tangent_normal_reflect = (float3(tangent_normal_reflect.x * bumpy_reflect, tangent_normal_reflect.y * bumpy_reflect, tangent_normal_reflect.z));
    float3 worldNorm_reflect = normalize(mul(tangent2World, tangent_normal_reflect).xyz);

    float3 lightReflected = reflect(lightVector, worldNorm_reflect);

    // compute the diffuse term
  	float3 diffuseLight = max(0.0, dot(-lightVector, normal));
	
	  // compute the specular term
    float3 specularLight;
    specularLight = pow(max(0.0, dot(-viewVector, lightReflected)), shininess); // modified phong

    float F_reflect = IN.texcoord4.w;
    float F_lighting = IN.texcoord5.w;

    float3 reflVector = reflect(viewVector, worldNorm_reflect);
    float3 reflColor = texCUBE(EnvironmentMap, reflVector);

    OUT.col.xyz = refl*F_reflect*reflColor + specular*F_lighting*specularLight + col*color + diffuse*diffuseLight;

    return OUT;
}

float4 LightPosition : Position = {1500.0f, 1500.0f, 1500.0f, 1.0f};
float4 Color = {1.0f, 0.0f, 0.0f, 1.0f};

float Min
<
	string gui = "slider";
	float uimin = 0.0;
	float uimax = 1.0;
	float uistep = 0.01;
> = 0.0;

float Max
<
	string gui = "slider";
	float uimin = 0.0;
	float uimax = 1.0;
	float uistep = 0.01;
> = 1.0;

float FresnelExponent
<
	string gui = "slider";
	float uimin = 0.0;
	float uimax = 100.0;
	float uistep = 1.0;
> = 1.0;

float Shininess
<
	string gui = "slider";
	float uimin = 0.0;
	float uimax = 100.0;
	float uistep = 1.0;
> = 8;

float reflColor
<
	string gui = "slider";
	float uimin = 0.0;
	float uimax = 5.0;
	float uistep = 0.1;
> = 0.5;

float diffuseLight
<
	string gui = "slider";
	float uimin = 0.0;
	float uimax = 5.0;
	float uistep = 0.1;
> = 0.5;

float specularLight
<
	string gui = "slider";
	float uimin = 0.0;
	float uimax = 5.0;
	float uistep = 0.1;
> = 1.5;

float Col
<
	string gui = "slider";
	float uimin = 0.0;
	float uimax = 5.0;
	float uistep = 0.1;
> = 0.5;

texture cubeMap : EnvMap
<
    string File = "nvlobby_new.dds";
    string TextureType = "Cube";
>;

samplerCUBE environmentMapSampler = sampler_state
{
	Texture = <cubeMap>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
};

texture normalMap_reflect_ : NormalMap1
<
	  string File = "wavy_normal.dds";
    string TextureType = "2D";
>;

sampler2D normalMap_reflect = sampler_state
{
	Texture = <normalMap_reflect_>;

	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;

      AddressU  = Clamp;
      AddressV  = Clamp;
      AddressW  = Clamp;
};

float Bumpy_reflect
<
	string gui = "slider";
	float uimin = 0.0;
	float uimax = 3.0;
	float uistep = 0.01;
> = 0.1;


technique fresnel
{
	pass p0 
	{
    VertexShader = compile vs_2_0 mainVS(wvp, world, worldI, viewI, view, Min, Max, FresnelExponent, LightPosition);

		ZEnable      = true;
		ZWriteEnable = true;
    //CullMode     = None;
		CullMode = CW;

    PixelShader = compile ps_2_0 fresnel(environmentMapSampler, normalMap_reflect, Bumpy_reflect, Color, Shininess, reflColor, diffuseLight, specularLight, Col);
	}
}
