// Copyright NVIDIA Corporation 2002-2004
// TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
// *AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
// BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES 

#pragma once
/** @file */

#include "nvsgcommon.h"

#include  "nvsg/Object.h"

namespace nvsg
{
  //! Class defining the common interface of an animation.
  /** An Animation is a pure virtual template class derived from Object that defines the common interface of an animation.
    */
  template<typename T> class Animation : public Object
  {
    public:
      //--  Functions specific to Animation --
      //! Interface for creating a clone of this animation.
      /** \returns pointer to the new animation */
      virtual const Animation<T> * clone( void ) const = 0;

      //! Get the number of frames in the animation.
      /** \returns number of animation frames */
      virtual size_t getNumberOfFrames( void ) const = 0;

      //! Random access operator for more convenient usage, constant version only.
      /** \returns constant reference to value of frame \a i  */
      virtual const T & operator[]( size_t i ) const = 0;

      //--  Functions reimplemented from Object  --
      virtual DataID getDataID( void ) const;
      virtual bool isDataShared( void ) const;

    protected:
      //! Protected constructor
      Animation( void );

      //! Protected destructor to prevent explicit creation on stack.
      virtual ~Animation( void );
  };

  template<typename T> Animation<T>::Animation( void )
  {
    __TRACE();
  }

  template<typename T> Animation<T>::~Animation( void )
  {
    __TRACE();
  }

  template<typename T> DataID Animation<T>::getDataID( void ) const
  {
    __TRACE();
    return( (DataID)this );
  }

  template<typename T> bool Animation<T>::isDataShared( void ) const
  {
    __TRACE();
    return( false );
  }
} //  namespace nvsg
