// Copyright NVIDIA Corporation 2002-2004
// TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
// *AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
// BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES 

#pragma once
/** \file */

#include "nvsgcommon.h"

#include "nvsg/Camera.h"

namespace nvsg
{
  //! A monoscopic camera
  /** This camera class is used to group the monoscopic camera types (like SimpleCamera, JitterCamera). */
  class MonoCamera : public Camera
  {
    public:
      //! Interface for cloning a MonoCamera.
      /** \returns  pointer to the new MonoCamera. */
      NVSG_API virtual const MonoCamera * clone( void ) const = 0;

      //! Interface for getting the projection matrix
      /** \returns  The projection matrix */
      NVSG_API virtual nvmath::Mat44f getProjection( void ) const = 0;
      
      //! Interface for getting the projection translation (used with stereo only).
      /** \returns  The projection translation in x */
      NVSG_API virtual float getProjectionDx( void ) const = 0;

      //! Interface for getting the inverse projection matrix
      /** \returns  The inverse projection matrix */
      NVSG_API virtual nvmath::Mat44f getInverseProjection( void ) const = 0;

      //! Interface for setting the project translation (used with stereo only).
      NVSG_API virtual void setProjectionDx( float dx   //!<  shift of projection in x-direction
                                           ) = 0;
     
  protected:
      //! Constructor
      NVSG_API MonoCamera( void );

      //! Protected destructor to prevent explicit creation on stack.
      NVSG_API virtual ~MonoCamera(void);
  };

} // namespace
