// Copyright NVIDIA Corporation 2002-2004
// TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
// *AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
// BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES 

#pragma once
/** \file */

#include "nvsgcommon.h"

#include "nvsg/SimpleCamera.h"

namespace nvsg
{
  //! A parallel camera
  /** This is a SimpleCamera that represents a parallel (non-perspective) camera.  */
  class ParallelCamera : public SimpleCamera
  {
    public:
      //! Create a ParallelCamera.
      /** \returns a constant pointer to a ParallelCamera. */
      NVSG_API static const ParallelCamera * create( void );

      //! Create a ParallelCamera by copying from a SimpleCamera.
      /** \returns a constant pointer to a ParallelCamera. */
      NVSG_API static const ParallelCamera * createFromBase( const SimpleCamera &rhs  //!< SimpleCamera to copy from
                                                           );

      //! Create a clone of this ParallelCamera.
      /** \returns a constant pointer to a ParallelCamera. */
      NVSG_API virtual const ParallelCamera * clone( void ) const;

      //! Get the projection matrix
      /** \returns  The projection matrix
        */
      NVSG_API virtual nvmath::Mat44f getProjection() const;

      //! Get the inverse projection matrix
      /** \returns  The inverse projection matrix
        */
      NVSG_API virtual nvmath::Mat44f getInverseProjection()  const;

    protected:
      //! Constructor
      NVSG_API ParallelCamera();

      //! Partial Constructor
      NVSG_API ParallelCamera( const SimpleCamera &rhs );

      //! Copy Constructor
      NVSG_API ParallelCamera( const ParallelCamera &rhs );

      //! Protected destructor to prevent explicit creation on stack.
      NVSG_API virtual ~ParallelCamera(void);
  };
} // namespace
