// Copyright NVIDIA Corporation 2002-2004
// TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
// *AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
// BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES 

#pragma once
/** \file */

#include <map>

#include "nvsgcommon.h"

#include "nvtraverser/Traverser.h"

namespace nvtraverser
{
  //! Traverser that replaces identity transforms by groups.
  /** Identical StateSets are recognized and linked to the same one.
    * Identical StateAttributes are recognized and linked to the same one.
    * This way, redundant objects are removed, thereby reducing storage space as well as run time. */
  class IdentityToGroupTraverser : public Traverser
  {
    public:
      //! Constructor
      NVSG_API IdentityToGroupTraverser( void );

    protected:
      //! Protected destructor to prevent instantiation of an IdentityToGroupTraverser on stack.
      NVSG_API virtual ~IdentityToGroupTraverser( void );

      //! Routine to handle a \link nvsg::AnimatedTransform AnimatedTransform \endlink object while traversing the scene graph.
      /** Explicitly go to Traverser::handleTransform() to prevent handling in handleTransform. */
      NVSG_API virtual void handleAnimatedTransform( const nvsg::AnimatedTransform *p //!< points to the currently visited AnimatedTransform object.
                                                   );

      //! Routine to handle a \link nvsg::Group Group \endlink object while traversing the scene graph.
      /** Replaces any Transform in it's list of children that is to be replaced by a Group. */
      NVSG_API virtual void handleGroup( const nvsg::Group *p   //!< points to the currently visited Group
                                       );

      //! Routine to handle a \link nvsg::Transform Transform \endlink object while traversing the scene graph.
      /** After traversing the Transform \a p, this handler compares it to the identity transform. If it is the identity,
        * all it's children are moved to a Group, the pair <p,Group> are stored for handling in handleGroup. */
      NVSG_API virtual void handleTransform( const nvsg::Transform *p   //!< points to the currently visited Transform object.
                                           );

    private:
      std::map<const nvsg::Node*,const nvsg::Group*>  m_transformToGroupMap;
  };
}
