// Copyright NVIDIA Corporation 2002-2004
// TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
// *AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
// BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES 

#pragma once
/** \file */

#include "nvsgcommon.h"

#include "nvtraverser/GLTraverser.h"

namespace nvtraverser
{
  //! \c GLTraverser internally used by the \c OcclusionCullTraverser.
  /** This \c GLTraverser performs the minimal GL operations to determine the number of rendered samples per \c Drawable.
    * That number is used to determine/adjust the CullCode of that \c Drawable. */
  class OcclusionCullGLTraverser : public GLTraverser
  {
    public:
      //! Constructor
      NVSG_API OcclusionCullGLTraverser(void);

      NVSG_API std::vector<GLuint> * getQueries( void );
#if defined( _DEBUG )
      NVSG_API std::vector<const nvsg::Drawable *> * getDrawables( void );
#endif

    protected:
      //! Destructor
      NVSG_API virtual ~OcclusionCullGLTraverser(void);

      NVSG_API virtual bool preTraverseRoot( const nvsg::SimpleCamera *p, const nvsg::Node *root );
      NVSG_API virtual void postTraverseRoot( const nvsg::SimpleCamera *p, const nvsg::Node *root );

      NVSG_API virtual void handleDirectedLight( const nvsg::DirectedLight *p);
      NVSG_API virtual void handlePointLight( const nvsg::PointLight *p );
      NVSG_API virtual void handleSpotLight( const nvsg::SpotLight *p );

      NVSG_API virtual void handleQuads( const nvsg::Quads *p );
      NVSG_API virtual void handleAnimatedQuads( const nvsg::AnimatedQuads * p );
      NVSG_API virtual void handleQuadStrips( const nvsg::QuadStrips *p );
      NVSG_API virtual void handleTriangles( const nvsg::Triangles *p );
      NVSG_API virtual void handleAnimatedTriangles( const nvsg::AnimatedTriangles * p );
      NVSG_API virtual void handleSkinnedTriangles( const nvsg::SkinnedTriangles * p );
      NVSG_API virtual void handleTriStrips( const nvsg::TriStrips *p );

      NVSG_API virtual void handleStateSet( const nvsg::StateSet *p );
      NVSG_API virtual void handleCgFx( const nvsg::CgFx *p );
      NVSG_API virtual void handleFaceAttribute( const nvsg::FaceAttribute *p );
      NVSG_API virtual void handleMaterial( const nvsg::Material *p );
      NVSG_API virtual void handleTextureAttribute( const nvsg::TextureAttribute *p );
      NVSG_API virtual void handleTextureAttributeItem( const nvsg::TextureAttributeItem * p, size_t tu );

      NVSG_API virtual void handleJitterCamera( const nvsg::JitterCamera *p, const nvsg::Node *root );
      NVSG_API virtual void handleStereoCamera( const nvsg::StereoCamera *p, const nvsg::Node *root );

      NVSG_API virtual void handleHighlightBox( const nvsg::HighlightBox *p, const nvsg::GeoSet *g );
      NVSG_API virtual void handleHighlightEdges( const nvsg::HighlightEdges *p, const nvsg::GeoSet *geoSet );

    private:
      GLuint genQueryId( const nvsg::Drawable *p );

    private:
      std::vector<GLuint>  m_queries;
#if defined( _DEBUG )
      std::vector<const nvsg::Drawable *> m_drawables;
#endif
  };

  inline std::vector<GLuint> * OcclusionCullGLTraverser::getQueries( void )
  {
    return( &m_queries );
  }

#if defined( _DEBUG )
  inline std::vector<const nvsg::Drawable *> * OcclusionCullGLTraverser::getDrawables( void )
  {
    return( &m_drawables );
  }
#endif
}
