// Copyright NVIDIA Corporation 2002-2004
// TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
// *AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
// BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES 

#pragma once
#include "nvsg/nvsgapi.h"

#include <qgl.h>
#include <qevent.h>

#include "nvgl/GLContext.h"
#include "nvui/RenderArea.h"


namespace nvui {

class TrackballCameraManipulator;

class QtRAWnd : public QGLWidget, public RenderArea
{
public:
  QtRAWnd(void);
  virtual ~QtRAWnd(void);

  // nvsg::RenderArea overloads
  virtual void destroy      ();
  virtual void triggerRedraw();
  virtual bool init         (RenderArea * shareArea = NULL);


protected:
 
  //QWidget overloads
  virtual void mousePressEvent  (QMouseEvent  * event);
  virtual void mouseReleaseEvent(QMouseEvent  * event);
  virtual void mouseMoveEvent   (QMouseEvent  * event);

  // QGLWidget overloads
  virtual void initializeGL();
  virtual void resizeGL( int w, int h );
  virtual void paintGL();  
  inline virtual void makeCurrent();
  
  GLXDrawable   m_drawable;
  GLXContext    m_hglrc;
  Display*      m_dpy;    
};

inline void QtRAWnd::makeCurrent()
{
  QGLWidget::makeCurrent();

  // Because of internal nvsg data structures,
  // we have to call our WGLMakeCurrent() wrapper every time, even if glut has already made the context current.  
  // (Also see WGLAttachContext()/WGLDetachContext() call in glutRenderArea.cpp!)
  if( m_hglrc )
  {
    nvgl::GLXMakeCurrent(m_dpy, m_drawable, m_hglrc);
  }  
}

}
